@@ -2,21 +2,24 @@ import logging
 from typing import Dict, Any, Tuple
 
 import torch as to
-from torch import nn
 from torch.nn.modules.module import Module
 from torch.optim.optimizer import Optimizer
 from torch.utils.data.dataloader import DataLoader
 
+from message_passing_nn.data.preprocessor import Preprocessor
 from message_passing_nn.utils.loss_function_selector import LossFunctionSelector
 from message_passing_nn.utils.optimizer_selector import OptimizerSelector
+from message_passing_nn.utils.model_selector import ModelSelector
 
 
 class ModelTrainer:
-    def __init__(self, model: nn.Module, device: str) -> None:
-        self.model = model
+    def __init__(self, preprocessor: Preprocessor, device: str, normalize: bool = False) -> None:
+        self.preprocessor = preprocessor
+        self.device = device
+        self.normalize = normalize
+        self.model = None
         self.loss_function = None
         self.optimizer = None
-        self.device = device
 
     def instantiate_attributes(self,
                                data_dimensions: Tuple,
@@ -25,6 +28,7 @@ class ModelTrainer:
         number_of_nodes = adjacency_matrix_size[0]
         number_of_node_features = node_features_size[1]
         fully_connected_layer_output_size = labels_size[0]
+        self.model = ModelSelector.load_model(configuration_dictionary['model'])
         self.model = self.model.of(time_steps=configuration_dictionary['time_steps'],
                                    number_of_nodes=number_of_nodes,
                                    number_of_node_features=number_of_node_features,
@@ -45,8 +49,12 @@ class ModelTrainer:
                                                       adjacency_matrix.to(self.device), \
                                                       labels.to(self.device)
             current_batch_size = self._get_current_batch_size(labels)
+            if self.normalize:
+                node_features = self.preprocessor.normalize(node_features, self.device)
+                labels = self.preprocessor.normalize(labels, self.device)
             self.optimizer.zero_grad()
-            outputs = self.model.forward(node_features, adjacency_matrix=adjacency_matrix,
+            outputs = self.model.forward(node_features,
+                                         adjacency_matrix=adjacency_matrix,
                                          batch_size=current_batch_size)
             loss = self.loss_function(outputs, labels)
             training_loss += self._do_backpropagate(loss, training_loss)
@@ -63,6 +71,9 @@ class ModelTrainer:
                     node_features, adjacency_matrix, labels_validation = node_features.to(self.device), \
                                                                          adjacency_matrix.to(self.device), \
                                                                          labels_validation.to(self.device)
+                    if self.normalize:
+                        node_features = self.preprocessor.normalize(node_features, self.device)
+                        labels_validation = self.preprocessor.normalize(labels_validation, self.device)
                     current_batch_size = self._get_current_batch_size(labels_validation)
                     outputs = self.model.forward(node_features, adjacency_matrix, current_batch_size)
                     loss = self.loss_function(outputs, labels_validation)
